﻿using System;
using System.Threading.Tasks;
using System.Threading.Tasks.Dataflow;
using Windows.UI.Xaml.Controls;

namespace VSMDataflowPart3
{
    public class UiUpdateBlock<T> : IPropagatorBlock<T, T>
    {
        private readonly ITargetBlock<T> _target;
        private readonly ISourceBlock<T> _source;

        public UiUpdateBlock(TextBlock element)
        {
            _target = new ActionBlock<T>(x => element.Text = x.ToString(),
                new ExecutionDataflowBlockOptions() 
                { TaskScheduler = TaskScheduler.FromCurrentSynchronizationContext() });
            _source = new BroadcastBlock<T>(x => x);
        }

        public DataflowMessageStatus OfferMessage(DataflowMessageHeader messageHeader,
            T messageValue, ISourceBlock<T> source, bool consumeToAccept)
        {
            return _target.OfferMessage(messageHeader, messageValue,
                source, consumeToAccept);
        }

        public void Complete()
        {
            _target.Complete();
        }

        public Task Completion
        {
            get { return _source.Completion; }
        }

        public void Fault(Exception exception)
        {
            _target.Fault(exception);
        }

        public T ConsumeMessage(DataflowMessageHeader messageHeader,
            ITargetBlock<T> target, out bool messageConsumed)
        {
            return _source.ConsumeMessage(messageHeader,
                target, out messageConsumed);
        }

        public IDisposable LinkTo(ITargetBlock<T> target, DataflowLinkOptions linkOptions)
        {
            return _source.LinkTo(target, linkOptions);
        }

        public void ReleaseReservation(DataflowMessageHeader messageHeader, ITargetBlock<T> target)
        {
            _source.ReleaseReservation(messageHeader, target);
        }

        public bool ReserveMessage(DataflowMessageHeader messageHeader, ITargetBlock<T> target)
        {
            return _source.ReserveMessage(messageHeader, target);
        }
    }
}
